@@ -1,4 +1,4 @@
-#' Extract the deviance from a fitted "abess" object.
+#' Extract the deviance from a fitted "\code{abess}" object.
 #'
 #' Similar to other deviance methods, 
 #' which returns deviance from a fitted "\code{abess}" object.
@@ -32,17 +32,17 @@ deviance.abess <- function(object,
       dev <- object[["tune.value"]]
     } else {
       dev <- object[["dev"]]
-      supp_size <- object[["support.size"]]
+      edf <- object[["edf"]]
       if (type == "aic") {
-        adjust_term <- supp_size
+        adjust_term <- 2
       } else if (type == "bic") {
-        adjust_term <- supp_size
+        adjust_term <- log(object[["nobs"]])
       } else if (type == "ebic") {
-        adjust_term <- supp_size
+        adjust_term <- (2 * log(object[["nvars"]]) + log((object[["nobs"]])))
       } else if (type == "gic") {
-        adjust_term <- supp_size
+        adjust_term <- log(object[["nvars"]]) * log(log(object[["nobs"]]))
       }
-      dev <- dev + adjust_term
+      dev <- dev + adjust_term * edf
     }
   }
   
